@@ -8,7 +8,8 @@ from .anime import Anime, AnimeSearchResults, AnimeList, AnimeRanking, Seasonal
 from .manga import Manga, MangaSearchResults, MangaList, MangaRanking
 from .forum import BoardCategory, ForumTopics, Discussion
 from .enums import (Field, AnimeListStatus, MangaListStatus, Season,
-                    AnimeRankingType, MangaRankingType, AnimeListSort, MangaListSort)
+                    AnimeRankingType, MangaRankingType, AnimeListSort, MangaListSort,
+                    SeasonalAnimeSort)
 from .utils import MISSING
 
 _log = logging.getLogger(__name__)
@@ -20,55 +21,78 @@ class Client:
         self.__client_id: str = client_id
         self._session: requests.Session = requests.Session()
         self._session.headers.update({'X-MAL-CLIENT-ID': self.__client_id})
-        self._search_limit: int = 10  # between 1 and 100
+        self._limit: int = 10
         self._anime_fields: List[Field] = Field.default_anime()
         self._manga_fields: List[Field] = Field.default_manga()
         self._include_nsfw: bool = False
 
     @property
-    def search_limit(self) -> int:
+    def limit(self) -> int:
         """Maximum number of results per page. Defaults to 10. Can be changed
-        with any integer between 1 and 100. If a number outside of this range
-        is given then the value is set to the closest value inside the range.
-        """
-        return self._search_limit
+        to any positive integer, it will be automatically adjusted to fit within
+        the API limits.
 
-    @search_limit.setter
-    def search_limit(self, value: int) -> None:
-        self._search_limit = value if 0 < value < 100 else 100
-        _log.info(f'parameter "search_limit" default value set to {self._search_limit}')
+        Raises:
+            ValueError: if a negative value is passed
+        """
+        return self._limit
+
+    @limit.setter
+    def limit(self, value: int) -> None:
+        if value < 0:
+            raise ValueError('limit must be a positive integer')
+        self._limit = int(value)
+        _log.info(f'parameter "search_limit" default value set to {self._limit}')
 
     @property
     def anime_fields(self) -> List[Field]:
-        """Current fields that are requested in a anime query. Change this to change the
-        fields that are requested.
-        To change fields only for one request you should pass them to the search instead.
+        """Fields that are requested in a anime query. This value is used in the following
+        methods:
+         - anime_search
+         - get_anime
+         - get_anime_list
+         - get_seasonal_anime
+         - get_anime_ranking
+
+        Changes to this value are applied to all subsequent requests. Invalid fields
+        are automatically ignored.
+        It is possible to override this value per request by specifying the `fields` parameter.
         """
         return self._anime_fields
 
     @anime_fields.setter
     def anime_fields(self, new_fields: Sequence[Union[Field, str]]) -> None:
-        self._anime_fields = Field.from_list(new_fields)
-        _log.info(f'parameter "anime_fields" default value set to {self.anime_fields}')
+        fields = Field.from_list(new_fields)
+        self._anime_fields = [f for f in fields if f.is_anime]
+        _log.info(f'parameter "anime_fields" default value set to {self._anime_fields}')
 
     @property
     def manga_fields(self) -> List[Field]:
-        """Current fields that are requested in a manga query. Change this to change the
-        fields that are requested.
-        To change fields only for one request you should pass them to the search instead.
+        """Fields that are requested in a manga query. This value is used in the following
+        methods:
+         - manga_search
+         - get_manga
+         - get_manga_list
+         - get_manga_ranking
+
+        Changes to this value are applied to all subsequent requests. Invalid fields
+        are automatically ignored.
+        It is possible to override this value per request by specifying the `fields` parameter.
         """
         return self._manga_fields
 
     @manga_fields.setter
     def manga_fields(self, new_fields: Sequence[Union[Field, str]]) -> None:
-        self._manga_fields = Field.from_list(new_fields)
-        _log.info(f'parameter "manga_fields" default value set to {self.manga_fields}')
+        fields = Field.from_list(new_fields)
+        self._manga_fields = [f for f in fields if f.is_manga]
+        _log.info(f'parameter "manga_fields" default value set to {self._manga_fields}')
 
     @property
     def include_nsfw(self) -> bool:
-        """Whether searches and lists include titles marked as nsfw. Defaults to False.
-        Can be also specified for each query using the corresponding keyword, overriding
-        this setting.
+        """Specifies whether to include results marked as nsfw. Defaults to False.
+
+        Changes to this value are applied to all subsequent requests.
+        It is possible to override this value per request by specifying the `nsfw` parameter.
         """
         return self._include_nsfw
 
@@ -91,7 +115,7 @@ class Client:
         method they are used for this query only.
 
         Args:
-            query: string used to search titles, minimum length 3 characters
+            query: string used to search titles, betwneen 3 and 64 characters
 
         Keyword args:
             limit: maximum number of results, needs to be between 1 and 100
@@ -101,8 +125,13 @@ class Client:
 
         Returns:
             AnimeSearchResults: iterable object containing the results
+
+        Raises:
+            ValueError: when the query is not between 3 and 64 characters
         """
-        parameters = self._build_search_parameters(
+        if len(query) > 64 or len(query) < 3:
+            raise ValueError('query parameter needs to be between 3 and 64 characters long')
+        parameters = self._build_parameters(
             Endpoint.ANIME, limit=limit, offset=offset, fields=fields, nsfw=include_nsfw)
         parameters['q'] = query
         url: str = Endpoint.ANIME.url
@@ -125,7 +154,7 @@ class Client:
         method they are used for this query only.
 
         Args:
-            query: string used to search titles, minimum length 3 characters
+            query: string used to search titles, betwneen 3 and 64 characters
 
         Keyword args:
             limit: maximum number of results, needs to be between 1 and 100
@@ -135,8 +164,13 @@ class Client:
 
         Returns:
             MangaSearchResults: iterable object containing the results
+
+        Raises:
+            ValueError: when the query is not between 3 and 64 characters
         """
-        parameters = self._build_search_parameters(
+        if len(query) > 64 or len(query) < 3:
+            raise ValueError('query parameter needs to be between 3 and 64 characters long')
+        parameters = self._build_parameters(
             Endpoint.MANGA, limit=limit, offset=offset, fields=fields, nsfw=include_nsfw)
         parameters['q'] = query
         url: str = Endpoint.MANGA.url
@@ -157,7 +191,7 @@ class Client:
         Returns:
             Anime: the anime object with all the details
         """
-        parameters = self._build_search_parameters(
+        parameters = self._build_parameters(
             Endpoint.ANIME, fields=fields)
         url: str = Endpoint.ANIME.url + '/' + self._get_as_id(id)
         response = self._request(url, params=parameters)
@@ -176,7 +210,7 @@ class Client:
         Returns:
             Manga: the anime object with all the details
         """
-        parameters = self._build_search_parameters(
+        parameters = self._build_parameters(
             Endpoint.ANIME, fields=fields)
         url: str = Endpoint.MANGA.url + '/' + self._get_as_id(id)
         response = self._request(url, params=parameters)
@@ -210,7 +244,7 @@ class Client:
         Returns:
             AnimeList: iterable with all the entries of the list
         """
-        parameters = self._build_list_paramenters(
+        parameters = self._build_parameters(
             Endpoint.USER_ANIMELIST, limit=limit, offset=offset, fields=fields, status=status, nsfw=include_nsfw, sort=sort)
         url = Endpoint.USER_ANIMELIST.url.replace('{username}', username)
         response = self._request(url, params=parameters)
@@ -244,7 +278,7 @@ class Client:
         Returns:
             MangaList: iterable with all the entries of the list
         """
-        parameters = self._build_list_paramenters(
+        parameters = self._build_parameters(
             Endpoint.USER_MANGALIST, limit=limit, offset=offset, fields=fields, status=status, nsfw=include_nsfw, sort=sort)
         url = Endpoint.USER_MANGALIST.url.replace('{username}', username)
         response = self._request(url, params=parameters)
@@ -259,6 +293,7 @@ class Client:
         limit: int = MISSING,
         offset: int = MISSING,
         fields: Sequence[Union[Field, str]] = MISSING,
+        sort: Union[SeasonalAnimeSort, str] = MISSING,
         include_nsfw: bool = MISSING
     ) -> Seasonal:
         """Returns the list of anime aired during a specific season.
@@ -276,14 +311,19 @@ class Client:
             limit: set the number of entries to retrieve, defaults to 10
             offset: get results at a certain offset from the start, defaults to 0
             fields: set which fields to get for each entry
+            sort: how to sort the results
             include_nsfw: include results marked as nsfw
 
         Returns:
             Seasonal: container for the results, sorted by score
         """
-        parameters = self._build_search_parameters(
+        parameters = self._build_parameters(
             Endpoint.ANIME_SEASONAL, limit=limit, offset=offset, fields=fields, nsfw=include_nsfw)
-        parameters['sort'] = 'anime_score'  # TODO: make it customizable
+        if sort is not MISSING:
+            if isinstance(sort, str):
+                parameters['sort'] = sort
+            else:
+                parameters['sort'] = sort.value
         url = f'{Endpoint.ANIME_SEASONAL}/{year}/{season}'
         response = self._request(url, params=parameters)
         data = response.json()
@@ -313,7 +353,7 @@ class Client:
         Raises:
             ValueError: ranking_type is invalid, check AnimeRankingType for all options
         """
-        parameters = self._build_search_parameters(
+        parameters = self._build_parameters(
             Endpoint.ANIME_RANKING, limit=limit, offset=offset, fields=fields)
         if isinstance(ranking_type, str):
             ranking_type = AnimeRankingType(ranking_type)
@@ -346,7 +386,7 @@ class Client:
         Raises:
             ValueError: ranking_type is invalid, check MangaRankingType for all options
         """
-        parameters = self._build_search_parameters(
+        parameters = self._build_parameters(
             Endpoint.MANGA_RANKING, limit=limit, offset=offset, fields=fields)
         if isinstance(ranking_type, str):
             ranking_type = MangaRankingType(ranking_type)
@@ -394,7 +434,7 @@ class Client:
         Raises:
             ValueError: no argument was specified
         """
-        parameters = self._build_topic_parameters(Endpoint.FORUM_TOPICS, query=query, board_id=board_id,
+        parameters = self._build_parameters(Endpoint.FORUM_TOPICS, query=query, board_id=board_id,
                                                   subboard_id=subboard_id, limit=limit, offset=offset,
                                                   topic_user_name=topic_user_name, user_name=user_name)
         url: str = Endpoint.FORUM_TOPICS.url
@@ -453,122 +493,87 @@ class Client:
             response.raise_for_status()  # TODO: handle error and possible retries
         return response
 
-    def _build_search_parameters(
-        self,
-        endpoint: Endpoint,
-        *,
-        limit: int = MISSING,
-        offset: int = MISSING,
-        fields: Sequence[Union[Field, str]] = MISSING,
-        nsfw: bool = MISSING
-    ) -> Dict[str, str]:
-        parameters: Dict[str, str] = {}
-        if limit is not MISSING:
-            parameters['limit'] = str(self._get_limit(endpoint, limit))
-        else:
-            parameters['limit'] = str(self._search_limit)
-        if offset is not MISSING:
-            parameters['offset'] = str(offset)
-        if fields is not MISSING:
-            parsed_fields = Field.from_list(fields)
-            if endpoint.is_anime:
-                parameters['fields'] = ','.join(
-                    [f.value for f in parsed_fields if f.is_anime])
-            else:
-                parameters['fields'] = ','.join(
-                    [f.value for f in parsed_fields if f.is_manga])
-        else:
-            if endpoint.is_anime:
-                parameters['fields'] = ','.join(
-                    [f.value for f in self._anime_fields])
-            else:
-                parameters['fields'] = ','.join(
-                    [f.value for f in self._manga_fields])
-        # nsfw overrides the value stored in self.include_nsfw
-        if nsfw is MISSING:
-            if self.include_nsfw:
-                parameters['nsfw'] = 'true'
-        elif nsfw:
-            parameters['nsfw'] = 'true'
-        return parameters
-
-    def _build_list_paramenters(
-        self,
-        endpoint: Endpoint,
-        *,
-        limit: int = MISSING,
-        offset: int = MISSING,
-        fields: Sequence[Union[Field, str]] = MISSING,
-        status: Union[AnimeListStatus, MangaListStatus, str] = MISSING,
-        sort: Union[AnimeListSort, MangaListSort, str] = MISSING,
-        nsfw: bool = MISSING
-    ) -> Dict[str, str]:
-        parameters: Dict[str, str] = {}
-        if limit is not MISSING:
-            parameters['limit'] = str(self._get_limit(endpoint, limit))
-        if offset is not MISSING:
-            parameters['offset'] = str(offset)
-        if fields is not MISSING:
-            parsed_fields = Field.from_list(fields)
-            if endpoint.is_anime:
-                parameters['fields'] = ','.join(
-                    [f.value for f in parsed_fields if f.is_anime])
-            else:
-                parameters['fields'] = ','.join(
-                    [f.value for f in parsed_fields if f.is_manga])
-            parameters['fields'] += ',list_status'
-        else:
-            parameters['fields'] = 'list_status'
-        if status is not MISSING:
-            if isinstance(status, str):
-                value = status
-            else:
-                value = status.value
-            parameters['status'] = value
-        if sort is not MISSING:
-            if isinstance(sort, str):
-                value = sort   # NOTE: default sort is by title
-            else:
-                value = sort.value
-            parameters['sort'] = value
-        # nsfw overrides the value stored in self.include_nsfw
-        if nsfw is MISSING:
-            if self.include_nsfw:
-                parameters['nsfw'] = 'true'
-        elif nsfw:
-            parameters['nsfw'] = 'true'
-        return parameters
-
-    def _build_topic_parameters(
+    def _build_parameters(
         self,
         endpoint: Endpoint,
         *,
         query: str = MISSING,
-        board_id: int = MISSING,
-        subboard_id: int = MISSING,
         limit: int = MISSING,
         offset: int = MISSING,
+        fields: Sequence[Union[Field, str]] = MISSING,
+        nsfw: bool = MISSING,
+        status: Union[AnimeListStatus, MangaListStatus, str] = MISSING,
+        sort: Union[AnimeListSort, MangaListSort, SeasonalAnimeSort, str] = MISSING,
+        board_id: int = MISSING,
+        subboard_id: int = MISSING,
         topic_user_name: str = MISSING,
         user_name: str = MISSING
     ) -> Dict[str, str]:
         parameters: Dict[str, str] = {}
-        if query is not MISSING:
-            parameters['q'] = query
-        if board_id is not MISSING:
-            parameters['board_id'] = str(board_id)
-        if subboard_id is not MISSING:
-            parameters['subboard_id'] = str(subboard_id)
-        if limit is not MISSING:
-            parameters['limit'] = str(self._get_limit(endpoint, limit))
-        if offset is not MISSING:
-            parameters['offset'] = str(offset)
-        if topic_user_name is not MISSING:
-            parameters['topic_user_name'] = topic_user_name
-        if user_name is not MISSING:
-            parameters['user_name'] = user_name
-        if not parameters:
-            raise ValueError(
-                'At least one parameter must be specified to search topics.')
+        if not endpoint.is_forum:
+            if query is not MISSING:
+                parameters['q'] = query
+            if limit is not MISSING:
+                parameters['limit'] = str(self._get_limit(endpoint, limit))
+            else:
+                parameters['limit'] = str(self._limit)
+            if offset is not MISSING:
+                parameters['offset'] = str(offset)
+            if fields is not MISSING:
+                parsed_fields = Field.from_list(fields)
+                if endpoint.is_anime:
+                    parameters['fields'] = ','.join(
+                        [f.value for f in parsed_fields if f.is_anime])
+                else:
+                    parameters['fields'] = ','.join(
+                        [f.value for f in parsed_fields if f.is_manga])
+                if endpoint.is_list:
+                    parameters['fields'] += ',list_status'
+            else:
+                if endpoint.is_anime:
+                    parameters['fields'] = ','.join(
+                        [f.value for f in self._anime_fields])
+                else:
+                    parameters['fields'] = ','.join(
+                        [f.value for f in self._manga_fields])
+                if endpoint.is_list:
+                    parameters['fields'] += ',list_status'
+            if status is not MISSING:
+                if isinstance(status, str):
+                    value = status
+                else:
+                    value = status.value
+                parameters['status'] = value
+            if sort is not MISSING:
+                if isinstance(sort, str):
+                    value = sort    # NOTE: how can i validate the string?
+                else:
+                    value = sort.value
+                parameters['sort'] = value
+            # nsfw overrides the value stored in self.include_nsfw
+            if nsfw is MISSING:
+                if self.include_nsfw:
+                    parameters['nsfw'] = 'true'
+            elif nsfw:
+                parameters['nsfw'] = 'true'
+        else:  # forum endpoint
+            if query is not MISSING:
+                parameters['q'] = query
+            if board_id is not MISSING:
+                parameters['board_id'] = str(board_id)
+            if subboard_id is not MISSING:
+                parameters['subboard_id'] = str(subboard_id)
+            if limit is not MISSING:
+                parameters['limit'] = str(self._get_limit(endpoint, limit))
+            if offset is not MISSING:
+                parameters['offset'] = str(offset)
+            if topic_user_name is not MISSING:
+                parameters['topic_user_name'] = topic_user_name
+            if user_name is not MISSING:
+                parameters['user_name'] = user_name
+            if not parameters:
+                raise ValueError(
+                    'At least one parameter must be specified to search topics.')
         return parameters
 
     def _get_limit(self, endpoint: Endpoint, value: int) -> int:
