@@ -134,6 +134,10 @@ class MangaListSort(BaseEnum):
     start_date = 'manga_start_date'
     id = 'manga_id'
 
+class SeasonalAnimeSort(BaseEnum):
+    score = 'anime_score'
+    num_list_users = 'anime_num_list_users'
+
 
 class Season(BaseEnum):
     winter = 'winter'
@@ -180,6 +184,7 @@ class Field(BaseEnum):
     popularity = 'popularity'
     num_list_users = 'num_list_users'
     num_scoring_users = 'num_scoring_users'  # 0 if not aired/published
+    num_favorites = 'num_favorites'
     nsfw = 'nsfw'
     created_at = 'created_at'     # 01-01-1970 in some cases, probably a default value
     updated_at = 'updated_at'
@@ -251,15 +256,13 @@ class Field(BaseEnum):
     @classmethod
     def from_list(cls, fields: Sequence[Union[str, Field]]) -> List[Field]:
         """Returns the list of fields from the corresponding string representation.
+        Ignores invalid strings.
 
         Args:
             fields: list of fields
 
         Returns:
             List[Field]: converted fields
-
-        Raises:
-            ValueError: one or more strings are not a valid field
         """
         result: List[Field] = []
         for field in fields:
